if __name__ == '__main__':
    from lsshu.oauth.model import Model, Engine
    from lsshu.demo.model import permission as permission_demo

    Model.metadata.create_all(Engine)  # 创建表结构

    APP_PERMISSIONS = [
        permission_demo
    ]

    from config import OAUTH_ADMIN_USERS
    from lsshu.internal.helpers import store_permissions, init_user_and_password

    store_permissions(APP_PERMISSIONS)  # 初始化权限
    init_user_and_password(OAUTH_ADMIN_USERS)  # 初始化授权用户



from typing import Optional, Union

from pydantic import BaseModel

from config import SCHEMAS_SUCCESS_CODE, SCHEMAS_SUCCESS_STATUS, SCHEMAS_SUCCESS_MESSAGE


class Schemas(BaseModel):
    """状态返回"""
    code: Optional[int] = SCHEMAS_SUCCESS_CODE
    status: Optional[str] = SCHEMAS_SUCCESS_STATUS
    message: Optional[str] = SCHEMAS_SUCCESS_MESSAGE
    data: Optional[Union[BaseModel, dict, list, str, None]] = None


class SchemasPaginate(BaseModel):
    """分页"""
    # has_next: bool  # 如果下一页存在，返回True
    # has_prev: bool  # 如果上一页存在，返回True
    items: list  # 当前页的数据列表
    # next_num: Union[int, None]  # 下一页的页码
    # prev_num: Union[int, None]  # 上一页的页码
    # page: Union[int, None]  # 当前页码
    pages: int  # 总页数
    # per_page: Union[int, None]  # 每页的条数
    total: int  # 总条数


class ModelScreenParams(BaseModel):
    """获取列表默认参数"""
    page: Optional[int] = 1
    limit: Optional[int] = 15
    where: Optional[Union[dict, list]] = None



from sqlalchemy import Column, String

from lsshu.internal.db import Model
from lsshu.internal.method import plural

name = plural(__name__.capitalize())
table_name = name.replace('.', '_')
permission = {"name": "Demo", "scope": name, "action": [{"name": "de", "scope": "de"}]}


class Models(Model):
    """ 模型 """
    __tablename__ = table_name
    name = Column(String(15), nullable=False, unique=True, comment="名称")



def hashids_encode(ids: int, **kwargs):
    """
    hashids 加密
    :param ids:
    :param kwargs:
    :return:
    """
    from hashids import Hashids
    hashids = Hashids(**kwargs)
    return hashids.encode(ids)


def hashids_decode(string: str, **kwargs):
    """
    hashids 解密
    :param string:
    :param kwargs:
    :return:
    """
    from hashids import Hashids
    hashids = Hashids(**kwargs)
    return hashids.decode(string)


def get_mac_address():
    """
    获取 本机 mac 地址
    :return:
    """
    import uuid
    mac = uuid.UUID(int=uuid.getnode()).hex[-12:]
    return ":".join([mac[e:e + 2] for e in range(0, 11, 2)])


def get_host_name():
    """
    获取主机名
    :return:
    """
    import socket
    return socket.getfqdn(socket.gethostname())


def get_host_ip():
    """
    获取IP地址
    :return:
    """
    import socket
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
    return ip


def get_client_ip(request):
    """
    获取请求的ip
    :param request:
    :return:
    """
    headers = request.headers
    addr = headers.get('http_x_forwarded_for', None)
    if not addr:
        addr = headers.get('http_client_ip', None)
    if not addr:
        addr = headers.get('remote_addr', None)
    if not addr:
        addr = headers.get('remote-host', None)
    if not addr:
        addr = headers.get('x-real-ip', None)
    if not addr:
        addr = headers.get('x-forwarded-for', None)
    if not addr:
        addr = headers.get('x-forwarded-for', None)
    return addr or None


def md5_string(string):
    """
    md5 字符串
    :param string:
    :return:
    """
    import hashlib
    md5hash = hashlib.md5(string.encode('utf8'))
    return md5hash.hexdigest()


def md5_file(file):
    """
    md5 文件
    :param file:
    :return:
    """
    import hashlib
    m = hashlib.md5()
    with open(file, 'rb') as obj:
        while True:
            data = obj.read(4096)
            if not data:
                break
            m.update(data)

    return m.hexdigest()


def str_bytes(string: str, **kwargs):
    """
    字符串转字节
    :param string:
    :param kwargs:
    :return:
    """
    return bytes(string, **kwargs)


def bytes_str(b: bytes, **kwargs):
    """
    字节转字符串
    :param b:
    :param kwargs:
    :return:
    """
    return str(b, **kwargs)


def plural(word: str):
    """
    字符串转复制
    :param word:
    :return:
    """
    if word.endswith('y'):
        return word[:-1] + "ies"
    elif word[-1] in 'sx' or word[-2:] in ["sh", "ch"]:
        return word + "es"
    elif word.endswith('an'):
        return word[:-2] + "en"
    else:
        return word + "s"



from typing import Union, List, Tuple

from pydantic import BaseModel
from sqlalchemy import or_
from sqlalchemy.orm import Session

from lsshu.internal.db import Model


class BaseCRUD(object):
    params_pk = "id"
    params_delete_tag = "deleted_at"  # 伪删除 字段
    params_model = Model  # 操作模型
    params_db = Session  # db实例
    params_query = None  # 查询实例
    params_relationship: dict = {}  # 多对多时使用
    params_action_method: list = [
        "start", "pseudo_deletion", "query", "where", "order",
        "page", "offset", "limit", "clear_params", "end"
    ]
    params: dict = {
        "where": [],  # 查询条件
        "order": [],  # 排序
    }

    @classmethod
    def init(cls, model, db=None, pk="id"):
        """
        初始化
        :param model:
        :param db:
        :param pk:
        :return:
        """
        cls.params_model = model
        cls.params_db = db
        cls.params_pk = pk
        return cls

    @classmethod
    def action_clear_params(cls, name: Union[str, list, tuple, None] = None):
        """
        清除参数条件
        :param name:
        :return:
        """
        if name and type(name) == str:
            if name in cls.params:
                del cls.params[name]
        elif name and (type(name) == list or type(name) == tuple):
            for n in name:
                if n in cls.params:
                    del cls.params[n]
        else:
            cls.params.clear()
        return cls

    @classmethod
    def db(cls, db):
        """
        设定 db
        :param db:
        :return:
        """
        cls.params_db = db
        cls.action_clear_params()
        return cls

    @classmethod
    def pseudo_deletion(cls, pseudo: bool):
        """
        设定是否 伪删除
        :param pseudo:
        :return:
        """
        cls._params_pseudo = pseudo
        return cls

    @classmethod
    def choose_pseudo_deletion(cls, choose: bool):
        """
        设定是否 伪删除操作 临时
        :param choose:
        :return:
        """
        cls._params_choose_pseudo = choose
        return cls

    @classmethod
    def all(cls, **kwargs):
        """
        多数据
        :param kwargs:
        :return:
        """
        data = cls.action_params(**kwargs).action().params_query.all()
        return data

    @classmethod
    def removed(cls, **kwargs):
        """
        伪删除数据
        :param kwargs:
        :return:
        """
        data = cls.choose_pseudo_deletion(False).where(cls.params_delete_tag, 'is_not', None).action_params(
            **kwargs).action().params_query.all()
        return data

    @classmethod
    def recovery(cls, **kwargs):
        """
        恢复 伪删除数据
        :param kwargs:
        :return:
        """
        data = cls.choose_pseudo_deletion(False).where(cls.params_delete_tag, 'is_not', None).action_params(
            **kwargs).action().params_query.update({cls.params_delete_tag: None})
        return data

    @classmethod
    def first(cls, **kwargs):
        """
        单条数据
        :param kwargs:
        :return:
        """
        data = cls.action_params(**kwargs).action().params_query.first()
        return data

    @classmethod
    def paginate(cls, **kwargs):
        """
        分页 操作
        :param kwargs:
        :return:
        """
        import math
        cls.action_params(**kwargs)
        page = cls.params.get('page', 1)
        limit = cls.params.get('limit', 0)

        items = cls.action().params_query.all()
        total = cls.action_clear_params(('limit', 'page', 'offset')).action().params_query.count()
        pages = math.ceil(total / limit) if type(total) is int and type(limit) is int and limit != 0 else 1
        next_num = page + 1 if pages > page else None
        prev_num = page - 1 if page > 1 else None
        return {
            "has_next": bool(next_num),  # 如果下一页存在，返回True
            "has_prev": bool(prev_num),  # 如果上一页存在，返回True
            "items": items,  # 当前页的数据列表
            "next_num": next_num,  # 下一页的页码
            "prev_num": prev_num,  # 上一页的页码
            "page": page,  # 当前页码
            "pages": pages,  # 总页数
            "per_page": limit,  # 每页的条数
            "total": total,  # 总条数
        }

    @classmethod
    def store(cls, db: Session, item: BaseModel, **kwargs):
        """
        创建模型数据
        :param db:
        :param item:
        :param kwargs:
        :return:
        """
        cls.action_params(db=db, **kwargs)
        # 处理关联
        import copy
        _item = copy.deepcopy(item)
        if type(cls.params_relationship) is dict:
            for relation in cls.params_relationship:
                if hasattr(item, relation) and bool(getattr(item, relation)):
                    delattr(item, relation)
        # 创建实例
        db_item = cls.params_model(**kwargs, **item.dict(exclude_unset=True))
        # 添加关联
        if type(cls.params_relationship) is dict:
            for (relation, relation_class) in cls.params_relationship.items():
                if hasattr(_item, relation) and bool(getattr(_item, relation)):
                    _relation = BaseCRUD.all(model=relation_class, db=db, where=(
                        "id", 'in_', getattr(_item, relation)))
                    setattr(db_item, relation, _relation)

        cls.params_db.add(db_item)
        cls.params_db.commit()
        cls.params_db.refresh(db_item)
        return db_item

    @classmethod
    def update(cls, **kwargs):
        """
        更新模型数据
        :param kwargs:
        :return:
        """
        item = cls._update_relationship(**kwargs)
        exclude_unset = kwargs.get('exclude_unset', False)
        cls.action_params(**kwargs).action().params_query.update(
            item.dict(exclude_unset=exclude_unset)), cls.params_db.commit(), cls.params_db.close()
        return cls.first(**kwargs)

    @classmethod
    def _update_relationship(cls, db: Session, item: BaseModel, **kwargs):
        """
        更新处理关联 多对多
        :param db:
        :param item:
        :return:
        """
        if type(cls.params_relationship) is dict:
            for (relation, relation_class) in cls.params_relationship.items():
                _obj = cls.first(db=db, **kwargs)
                if hasattr(item, relation) and bool(getattr(item, relation)):
                    _relation = BaseCRUD.all(db=db, model=relation_class, where=(
                        "id", 'in_', getattr(item, relation)))
                    setattr(_obj, relation, _relation)
                    db.add(_obj), db.commit(), db.close()
                elif hasattr(item, relation):
                    [getattr(_obj, relation).remove(rela) for rela in getattr(_obj, relation)]
                delattr(item, relation)
        return item

    @classmethod
    def delete(cls, **kwargs):
        """
        删除多模型数据
        :param kwargs:
        :return:
        """
        query = cls.action_params(**kwargs).action().params_query
        response = query.delete()
        cls.params_db.commit(), cls.params_db.close()
        return response

    @classmethod
    def action_query(cls):
        """
        获取查询实例
        :return:
        """
        cls.params_query = cls.params_db.query(cls.params_model)
        return cls

    @classmethod
    def pk(cls, pk: int):
        """
        设定查询 主键
        :param pk:
        :return:
        """
        cls.where(cls.params_pk, pk)
        return cls

    @classmethod
    def pks(cls, pks: Union[List[int], Tuple[int]]):
        """
        设定查询 多主键
        :param pks:
        :return:
        """
        pks = list(pks) if type(pks) == tuple else pks
        cls.where(cls.params_pk, 'in_', pks)
        return cls

    @classmethod
    def model(cls, model):
        """
        设定 操作模型
        :param model:
        :return:
        """
        cls.params_model = model
        return cls

    @classmethod
    def where(cls, where: Union[List[tuple], List[list], Tuple[tuple], Tuple[list], list, tuple, str], *args):
        """
        添加查询 条件
        :param where:
        :return:
        """
        _where = cls.params.get('where', [])
        if (type(where) == list or type(where) == tuple) and (type(where[0]) == list or type(where[0]) == tuple):
            _where.extend(where)
        elif (type(where) == list or type(where) == tuple) and type(where[0]) == str:
            _where.append(where)
        elif type(where) == str:
            _where.append((where, *args))
        cls.params.update({"where": _where})
        return cls

    @classmethod
    def action_where(cls):
        """
        过滤模型数据条件
        :return:
        """
        where = cls.params.get('where', None)
        if where and (type(where) == tuple or type(where) == list):  # 设置过滤 like
            for w in where:
                cls.filter_where(where=w) if w else None
        cls.params.update({"where": []})
        return cls

    @classmethod
    def filter_where(cls, where: Union[list, tuple, None] = None):
        """
        过滤数据条件
        :param where:
        :return:
        """
        query = cls.params_query
        if bool(where) and (type(where) == tuple or type(where) == list):
            if len(where) == 2:
                """('content', '西')"""
                query = query.filter(getattr(cls.params_model, where[0]) == where[1])
            elif len(where) == 3:
                if where[1] == "==" or where[1] == "=" or where[1] == "eq":
                    """('content', '==', '西')"""
                    query = query.filter(getattr(cls.params_model, where[0]) == where[2])
                elif where[1] == "!=" or where[1] == "<>" or where[1] == "><" or where[1] == "neq" or where[1] == "ne":
                    """('content', '!=', '西')"""
                    query = query.filter(getattr(cls.params_model, where[0]) != where[2])
                elif where[1] == ">" or where[1] == "gt":
                    """('content', '>', '西')"""
                    query = query.filter(getattr(cls.params_model, where[0]) > where[2])
                elif where[1] == ">=" or where[1] == "ge":
                    """('content', '>=', '西')"""
                    query = query.filter(getattr(cls.params_model, where[0]) >= where[2])
                elif where[1] == "<" or where[1] == "lt":
                    """('content', '<', '西')"""
                    query = query.filter(getattr(cls.params_model, where[0]) < where[2])
                elif where[1] == "<=" or where[1] == "le":
                    """('content', '<=', '西')"""
                    query = query.filter(getattr(cls.params_model, where[0]) <= where[2])
                elif where[1] in ["like", "ilike"]:
                    if not where[2] is None:
                        """('content', 'like', '西')"""
                        query = query.filter(
                            getattr(getattr(cls.params_model, where[0]), where[1])("%" + where[2] + "%")
                        )
                elif where[1] in ["or"]:
                    if type(where[0]) == tuple or type(where[0]) == list:
                        """(['name','content'], 'or', '西')"""
                        _filters = [(getattr(cls.params_model, fil) == where[2]) for fil in where[0]]
                        query = query.filter(or_(*_filters))

                    if type(where[0]) == str and (type(where[2]) == tuple or type(where[2]) == list):
                        """('name', 'or', ['西', '西'])"""
                        _filters = [(getattr(cls.params_model, where[0]) == fil) for fil in where[2]]
                        query = query.filter(or_(*_filters))
                elif where[1] in ["or_like", "or_ilike"]:
                    if type(where[0]) == tuple or type(where[0]) == list:
                        """(['name','content'], 'or_like', '西')"""
                        _filters = [(getattr(getattr(cls.params_model, fil), where[1][3:])("%" + where[2] + "%")) for
                                    fil in
                                    where[0]]
                        query = query.filter(or_(*_filters))
                    if type(where[0]) == str and (type(where[2]) == tuple or type(where[2]) == list):
                        """('name', 'or_like', ['西', '西'])"""
                        _filters = [(getattr(getattr(cls.params_model, where[0]), where[1][3:])("%" + fil + "%")) for
                                    fil in
                                    where[2]]
                        query = query.filter(or_(*_filters))
                else:
                    query = query.filter(getattr(getattr(cls.params_model, where[0]), where[1])(where[2]))
        cls.params_query = query
        return cls

    @classmethod
    def order(cls, order: Union[List[tuple], List[list], Tuple[tuple], Tuple[list], list, tuple]):
        """
        排序
        :param order:
        :return:
        """
        _order = cls.params.get('order', [])
        if type(order[0]) == list or type(order[0]) == tuple:
            _order.extend(order)
        elif type(order[0]) == str:
            _order.append(order)
        cls.params.update({"order": _order})
        return cls

    @classmethod
    def action_order(cls):
        """
        处理排序到查询
        :return:
        """
        query = cls.params_query
        orders = cls.params.get('order', [])
        if bool(orders):
            if orders and (type(orders) == tuple or type(orders) == list):  # 设置过滤 like
                for attr_item in orders:
                    query = query.order_by(getattr(getattr(cls.params_model, attr_item[0]), attr_item[1])())
        cls.params_query = query
        return cls

    @classmethod
    def page(cls, page: int):
        """
        设置page页
        :param page:
        :return:
        """
        cls.params.update({"page": page})
        return cls

    @classmethod
    def action_page(cls):
        """
        处理分页到查询
        :return:
        """
        query = cls.params_query
        page = cls.params.get('page', None)
        limit = cls.params.get('limit', None)
        if bool(page) and type(page) is int and bool(limit):
            offset = (page - 1) * limit
            query = query.offset(offset)

        cls.params_query = query
        return cls

    @classmethod
    def offset(cls, offset: int):
        """
        :param offset:
        :return:
        """
        cls.params.update({"offset": offset})
        return cls

    @classmethod
    def action_offset(cls):
        """
        :return:
        """
        query = cls.params_query
        offset = cls.params.get('offset', None)
        limit = cls.params.get('limit', None)
        if bool(offset) and bool(limit):
            query = query.offset(offset)
        cls.params_query = query
        return cls

    @classmethod
    def limit(cls, limit: int):
        """
        :param limit:
        :return:
        """
        cls.params.update({"limit": limit})
        return cls

    @classmethod
    def action_limit(cls):
        """
        :return:
        """
        query = cls.params_query
        limit = cls.params.get('limit', None)
        offset = cls.params.get('offset', None)
        page = cls.params.get('page', None)
        if bool(limit) and (bool(offset) or bool(page)):
            query = query.limit(limit)
        cls.params_query = query
        return cls

    @classmethod
    def action(cls):
        """处理查询参数 params 到 query"""
        [(getattr(cls, "action_%s" % action)() if hasattr(cls, "action_%s" % action) else None) for action in
         cls.params_action_method]
        return cls

    @classmethod
    def screen_params(cls, params: BaseModel):
        cls.action_params(**params.dict())
        return cls

    @classmethod
    def action_params(cls, **kwargs):
        """
        处理kwargs 参数到 params
        :param kwargs:
        :return:
        """
        [(getattr(cls, params)(params_value) if hasattr(cls, params) else None) for params, params_value in
         kwargs.items()]
        return cls

    @classmethod
    def update_or_store_model(cls, **kwargs):
        """
        更新或者创建
        :param kwargs:
        :return:
        """
        instance = cls.first(**kwargs)
        if instance:
            return cls.update(**kwargs, pk=getattr(instance, cls.params_pk), exclude_unset=True)
        else:
            del kwargs['where']
            return cls.store(**kwargs)

    @classmethod
    def find_or_store_model(cls, **kwargs):
        """
        查找或者创建
        """
        instance = cls.first(**kwargs)
        if not instance:
            del kwargs['where']
            return cls.store(**kwargs)
        return instance


class CRUDTree(BaseCRUD):
    @classmethod
    def move_inside(cls, db: Session, inside_id: int, **kwargs):
        """
        移动到 inside_id 下
        :param db:
        :param inside_id:
        :param kwargs:
        :return:
        """
        node = cls.first(db=db, **kwargs)
        return node.move_inside(inside_id)

    @classmethod
    def move_after(cls, db: Session, after_id: int, **kwargs):
        """
        移动到 after_id 后
        :param db:
        :param after_id:
        :param kwargs:
        :return:
        """
        node = cls.first(db=db, **kwargs)
        return node.move_after(after_id)

    @classmethod
    def get_tree(cls, db: Session, json=False, json_fields=None, query=None):
        """
        获取树
        :param db:
        :param json:
        :param json_fields:
        :param query:
        :return:
        """

        def query_fun(nodes):
            return nodes.filter(getattr(cls.params_model, cls.params_delete_tag).is_(None))

        def json_fields_fun(node):
            import copy
            _node = copy.deepcopy(node)
            delattr(_node, cls.params_delete_tag)
            return {"id": _node.id, "label": _node.name, "node": _node}

        return cls.params_model.get_tree(session=db, json=json, json_fields=json_fields_fun if json is True and not json_fields else json_fields,
                                         query=query if query else query_fun)



from datetime import datetime
from typing import Union

from sqlalchemy import create_engine, Column, Integer, TIMESTAMP, or_
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from config import DB_SQLALCHEMY_DATABASE_URL, DB_ENGINE_KWARGS, DB_SESSION_MAKER_KWARGS

Engine = create_engine(
    DB_SQLALCHEMY_DATABASE_URL,
    **DB_ENGINE_KWARGS
)

SessionLocal = sessionmaker(bind=Engine, **DB_SESSION_MAKER_KWARGS)
# 创建基本映射类
Base = declarative_base(bind=Engine, name='Model')


def dbs():
    """
    实例 sessionmaker
    :return:
    """
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


class Model(Base):
    __abstract__ = True
    id = Column(Integer, primary_key=True, index=True, autoincrement=True)
    created_at = Column(TIMESTAMP, nullable=True, default=datetime.now, comment="创建日期")
    updated_at = Column(TIMESTAMP, nullable=True, default=datetime.now, onupdate=datetime.now, comment="更新日期")

    def to_dict(self):
        """
        ORM转dict
        :return:
        """
        return {c.name: getattr(self, c.name, None) for c in self.__table__.columns}



from typing import Optional, Union

from fastapi import Depends, Security
from fastapi.security import SecurityScopes, OAuth2PasswordBearer
from sqlalchemy.orm import Session

from config import OAUTH_TOKEN_URL, OAUTH_TOKEN_SCOPES, OAUTH_SECRET_KEY, OAUTH_ALGORITHM, OAUTH_LOGIN_SCOPES
from lsshu.internal.db import dbs
from lsshu.internal.helpers import token_payload
from lsshu.internal.schema import ModelScreenParams
from lsshu.oauth.user.crud import CRUDOAuthUser
from lsshu.oauth.user.schema import SchemasOAuthUser, SchemasOAuthScopes


def model_screen_params(page: Optional[int] = 1, limit: Optional[int] = 25, where: Optional[Union[dict, list]] = None):
    """列表筛选参数"""
    return ModelScreenParams(page=page, limit=limit, where=where)


oauth2_scheme = OAuth2PasswordBearer(tokenUrl=OAUTH_TOKEN_URL, scopes=OAUTH_TOKEN_SCOPES)


async def current_user_security(security_scopes: SecurityScopes, token: str = Depends(oauth2_scheme)):
    """
    解析加密字段
    :param security_scopes:
    :param token:
    :return:
    """
    payload = token_payload(security_scopes, token, OAUTH_SECRET_KEY, OAUTH_ALGORITHM)
    """处理授权用户实时情况"""
    # Todo
    """处理授权用户实时情况"""
    return SchemasOAuthUser(**payload)


async def auth_user(auth: SchemasOAuthUser = Security(current_user_security, scopes=[OAUTH_LOGIN_SCOPES]),
                    db: Session = Depends(dbs)):
    """
    demo
    :param auth:
    :param db:
    :return:
    """
    user = CRUDOAuthUser.first(db=db, pk=auth.user_id)
    return SchemasOAuthScopes(user=user, scopes=auth.scopes)



# API 接口返回数据
SCHEMAS_SUCCESS_CODE = 0
SCHEMAS_SUCCESS_STATUS = 'success'
SCHEMAS_SUCCESS_MESSAGE = '数据请求成功！'
SCHEMAS_ERROR_CODE = 1
SCHEMAS_ERROR_STATUS = 'error'
SCHEMAS_ERROR_MESSAGE = '数据请求失败！'

# 站点
HOST_URL = ""

# 上传目录
UPLOAD_NAME = "static"
UPLOAD_DIR = "static"
UPLOAD_URI = "/static"

# OAuth 授权相关
OAUTH_DEFAULT_TAGS = ['OAuth']
OAUTH_LOGIN_SCOPES = "login"

OAUTH_TOKEN_URL = "/api/token"
OAUTH_TOKEN_SCOPES = {
    OAUTH_LOGIN_SCOPES: OAUTH_LOGIN_SCOPES.capitalize()
}
OAUTH_SECRET_KEY = "4a876f7766d1a0e9d97231089be927e38d6dea09233ad212f056b7f1a75cd41d"
OAUTH_ALGORITHM = "HS256"
OAUTH_ACCESS_TOKEN_EXPIRE_MINUTES = 300
OAUTH_OAUTH_ROUTER = {}

# 超级管理员 账号:密码
OAUTH_ADMIN_USERS = {
    "admin": "admin"
}

# 数据库相关
import os

DB_SQLALCHEMY_DATABASE_URL = "sqlite:///{}".format(os.path.join(os.path.dirname(__file__), 'db.sqlite3'))
# DB_SQLALCHEMY_DATABASE_URL = "mysql+pymysql://root:root@192.168.1.3:3306/ic"

# echo=True表示引擎将用repr()函数记录所有语句及其参数列表到日志
# 由于SQLAlchemy是多线程，指定check_same_thread=False来让建立的对象任意线程都可使用。这个参数只在用SQLite数据库时设置
DB_ENGINE_KWARGS = {
    # "echo": True,
    # "encoding": 'utf-8',
    # "pool_pre_ping": True,
    # "pool_size": 100,
    # "pool_recycle": 3600,
    # "max_overflow": 100,
    "connect_args": {
        'check_same_thread': False,
        # "charset": "utf8mb4"
    }
}
# 在SQLAlchemy中，CRUD都是通过会话(session)进行的，所以我们必须要先创建会话，每一个SessionLocal实例就是一个数据库session
# flush()是指发送数据库语句到数据库，但数据库不一定执行写入磁盘；commit()是指提交事务，将变更保存到数据库文件
DB_SESSION_MAKER_KWARGS = {
    "autoflush": False,
    "autocommit": False,
    "expire_on_commit": True
}



from typing import List

from fastapi import APIRouter, Depends, Security, HTTPException, status
from fastapi.security import OAuth2PasswordRequestForm
from sqlalchemy.orm import Session

from config import OAUTH_OAUTH_ROUTER, OAUTH_DEFAULT_TAGS, OAUTH_ACCESS_TOKEN_EXPIRE_MINUTES, OAUTH_LOGIN_SCOPES, OAUTH_ADMIN_USERS, OAUTH_SECRET_KEY, OAUTH_ALGORITHM
from lsshu.internal.db import dbs
from lsshu.internal.depends import model_screen_params, auth_user
from lsshu.internal.helpers import token_access_token, token_verify_password
from lsshu.internal.schema import Schemas, ModelScreenParams
from lsshu.oauth.model import user_name
from lsshu.oauth.permission.crud import CRUDOAuthPermission
from lsshu.oauth.role.crud import CRUDOAuthRole
from lsshu.oauth.user.crud import CRUDOAuthUser
from lsshu.oauth.user.schema import SchemasOAuthScopes, SchemasLoginResponse, SchemasLogin, SchemasOAuthUserMeStatusResponse, SchemasPaginateItem, SchemasParams, \
    SchemasOAuthUserResponse, SchemasOAuthUserStoreUpdate

tags = OAUTH_DEFAULT_TAGS + ['User']
router = APIRouter(tags=tags)
user_scopes = [user_name, ]


def authenticate_user(db: Session, username: str, password: str):
    """
    验证用户信息
    :param db:
    :param username:
    :param password:
    :return:
    """
    from lsshu.oauth.user.crud import CRUDOAuthUser
    user = CRUDOAuthUser.first(db=db, where=("username", username))

    if not user or not token_verify_password(plain_password=password, hashed_password=user.password):
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST, detail="Incorrect username or password")
    return user


def token_authenticate_access_token(db, username: str, password: str, scopes: list) -> str:
    """
    认证用户且生成用户token
    :param db:
    :param username:
    :param password:
    :param scopes:
    :return:
    """
    from datetime import timedelta
    user = authenticate_user(db=db, username=username, password=password)
    access_token_expires = timedelta(minutes=int(OAUTH_ACCESS_TOKEN_EXPIRE_MINUTES))
    scopes = scopes + [OAUTH_LOGIN_SCOPES]
    """处理用户拥有的权限"""
    # Todo user.roles user.permission
    if user.username in OAUTH_ADMIN_USERS:
        from lsshu.oauth.permission.crud import CRUDOAuthPermission
        permissions = CRUDOAuthPermission.all(db=db)
        scopes = scopes + [permission.scope for permission in permissions]
    else:
        # 获取用户权限
        scopes = scopes + [permission.scope for permission in user.permissions]
        # 获取用户角色权限
        scopes = scopes + [scope for role in user.roles for scope in (role.scopes.split(' '))]
    """处理用户拥有的权限"""
    return token_access_token(
        data={"sub": user.username, "user_id": user.id, "scopes": list(set(scopes))},
        key=OAUTH_SECRET_KEY,
        algorithm=OAUTH_ALGORITHM,
        expires_delta=access_token_expires
    )


@router.post('/token')
async def login_for_access_token(db: Session = Depends(dbs), form_data: OAuth2PasswordRequestForm = Depends()):
    """
    获取登录授权:
    - **form_data**: 登录数据
    """
    access_token = token_authenticate_access_token(
        db=db,
        username=form_data.username,
        password=form_data.password,
        scopes=form_data.scopes
    )
    return SchemasLoginResponse(data=SchemasLogin(access_token=access_token, token_type="bearer"))


@router.get('/me')
async def get_login_access_me(auth: SchemasOAuthScopes = Security(auth_user)):
    """
    获取登录授权:
    """
    return SchemasOAuthUserMeStatusResponse(data=auth.user, scopes=auth.scopes)


@router.get('/{}'.format(user_name), name="get {}".format(user_name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.list" % user_name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUDOAuthUser.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasPaginateItem(**db_model_list))


@router.get('/{}.params'.format(user_name), name="get {}".format(user_name))
async def params_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.list" % user_name])):
    """
    :param db:
    :param auth:
    :return:
    """
    data = {
        "roles": CRUDOAuthRole.all(db=db),
        "permissions": CRUDOAuthPermission.all(db=db)
    }

    return Schemas(data=SchemasParams(**data))


@router.get('/{}/{{pk}}'.format(user_name), name="get {}".format(user_name))
async def get_model(pk: int, db: Session = Depends(dbs),
                    auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.get" % user_name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthUser.first(db=db, where=(CRUDOAuthUser.params_pk, pk))
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="{} not found".format(user_name.capitalize()))
    return Schemas(data=SchemasOAuthUserResponse(**db_model.to_dict()))


@router.post('/{}'.format(user_name), name="get {}".format(user_name))
async def store_model(item: SchemasOAuthUserStoreUpdate, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.store" % user_name])):
    """
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthUser.first(db=db, where=("username", item.username))
    if db_model is not None:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                            detail="{} already registered".format(user_name.capitalize()))
    bool_model = CRUDOAuthUser.store(db=db, item=item)
    return Schemas(data=SchemasOAuthUserResponse(**bool_model.to_dict()))


@router.put("/{}/{{pk}}".format(user_name), name="update {}".format(user_name))
async def update_put_model(pk: int, item: SchemasOAuthUserStoreUpdate, db: Session = Depends(dbs),
                           auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.update" % user_name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthUser.first(db=db, where=(CRUDOAuthUser.pk, pk))
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="{} not found".format(user_name.capitalize()))
    bool_model = CRUDOAuthUser.update(db=db, pk=pk, item=item)
    return Schemas(data=SchemasOAuthUserResponse(**bool_model.to_dict()))


@router.patch("/{}/{{pk}}".format(user_name), name="update {}".format(user_name))
async def update_patch_model(pk: int, item: SchemasOAuthUserStoreUpdate, db: Session = Depends(dbs),
                             auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.update" % user_name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthUser.first(db=db, where=(CRUDOAuthUser.pk, pk))
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="{} not found".format(user_name.capitalize()))
    bool_model = CRUDOAuthUser.update(db=db, pk=pk, item=item, exclude_unset=True)
    return Schemas(data=SchemasOAuthUserResponse(**bool_model.to_dict()))


@router.delete("/{}/{{pk}}".format(user_name), name="delete {}".format(user_name))
async def delete_model(pk: int, db: Session = Depends(dbs),
                       auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.delete" % user_name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthUser.delete(db=db, pk=pk)
    return Schemas(data=bool_model)


@router.delete("/{}".format(user_name), name="deletes {}".format(user_name))
async def delete_models(pks: List[int], db: Session = Depends(dbs),
                        auth: SchemasOAuthScopes = Security(auth_user, scopes=user_scopes + ["%s.delete" % user_name])):
    """
    :param pks:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthUser.delete(db=db, pks=pks)
    return Schemas(data=bool_model)



from typing import List

from fastapi import APIRouter, Depends, Security, HTTPException, status
from sqlalchemy.orm import Session

from config import OAUTH_DEFAULT_TAGS
from lsshu.internal.db import dbs
from lsshu.internal.depends import model_screen_params, auth_user
from lsshu.internal.schema import ModelScreenParams, Schemas
from lsshu.oauth.model import permission_name
from lsshu.oauth.permission.crud import CRUDOAuthPermission
from lsshu.oauth.permission.schema import SchemasOAuthPermissionPaginateItem, SchemasOAuthPermissionTreeStatusResponse, SchemasOAuthPermissionResponse, \
    SchemasOAuthPermissionStoreUpdate
from lsshu.oauth.user.schema import SchemasOAuthScopes

tags = OAUTH_DEFAULT_TAGS + ['Permission']
router = APIRouter(tags=tags)
permission_scopes = [permission_name, ]


@router.get('/{}'.format(permission_name), name="get {}".format(permission_name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.list" % permission_name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUDOAuthPermission.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasOAuthPermissionPaginateItem(**db_model_list))


@router.get('/{}.params'.format(permission_name), name="get {}".format(permission_name))
async def params_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.list" % permission_name])):
    """
    :param db:
    :param auth:
    :return:
    """
    return Schemas(data={})


@router.get('/{}.tree'.format(permission_name), name="get {}".format(permission_name))
async def tree_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.list" % permission_name])):
    """
    :param db:
    :param auth:
    :return:
    """

    def json_fields(node):
        return node.to_dict()

    db_model_list = CRUDOAuthPermission.get_tree(db=db, json=True, json_fields=json_fields)
    return SchemasOAuthPermissionTreeStatusResponse(data=db_model_list)


@router.get('/{}.menus'.format(permission_name), name="get {}".format(permission_name))
async def menus_permission_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user)):
    """
    :param db:
    :param auth:
    :return:
    """

    def query_fun(nodes):
        return nodes.filter(getattr(CRUDOAuthPermission.params_model, CRUDOAuthPermission.params_delete_tag).is_(None)).filter(
            getattr(CRUDOAuthPermission.params_model, 'is_menu').is_(True)).filter(getattr(CRUDOAuthPermission.params_model, 'scope').in_(auth.scopes))

    db_model_list = CRUDOAuthPermission.get_tree(db=db, json=True, query=query_fun)
    return SchemasOAuthPermissionTreeStatusResponse(data=db_model_list)


@router.get('/{}/{{pk}}'.format(permission_name), name="get {}".format(permission_name))
async def get_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.get" % permission_name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthPermission.first(db=db, pk=pk)
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,
                            detail="{} not found".format(permission_name.capitalize()))
    return Schemas(data=SchemasOAuthPermissionResponse(**db_model.to_dict()))


@router.post('/{}'.format(permission_name), name="get {}".format(permission_name))
async def store_model(item: SchemasOAuthPermissionStoreUpdate, db: Session = Depends(dbs),
                      auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.store" % permission_name])):
    """
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthPermission.first(db=db, where=("name", item.name))
    if db_model is not None:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                            detail="{} already registered".format(permission_name.capitalize()))
    bool_model = CRUDOAuthPermission.store(db=db, item=item)
    return Schemas(data=SchemasOAuthPermissionResponse(**bool_model.to_dict()))


@router.put("/{}/{{pk}}".format(permission_name), name="update {}".format(permission_name))
async def update_put_model(pk: int, item: SchemasOAuthPermissionStoreUpdate, db: Session = Depends(dbs),
                           auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.update" % permission_name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthPermission.first(db=db, pk=pk)
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,
                            detail="{} not found".format(permission_name.capitalize()))
    bool_model = CRUDOAuthPermission.update(db=db, pk=pk, item=item)
    return Schemas(data=SchemasOAuthPermissionResponse(**bool_model.to_dict()))


@router.patch("/{}/{{pk}}".format(permission_name), name="update {}".format(permission_name))
async def update_patch_model(pk: int, item: SchemasOAuthPermissionStoreUpdate, db: Session = Depends(dbs),
                             auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.update" % permission_name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthPermission.first(db=db, pk=pk)
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND,
                            detail="{} not found".format(permission_name.capitalize()))
    bool_model = CRUDOAuthPermission.update(db=db, pk=pk, item=item, exclude_unset=True)
    return Schemas(data=SchemasOAuthPermissionResponse(**bool_model.to_dict()))


@router.patch("/{}/{{pk}}/move_inside".format(permission_name), name="update {}".format(permission_name))
async def move_inside_permission_model(pk: int, inside_id: int, db: Session = Depends(dbs),
                                       auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.update" % permission_name])):
    """
    移动到 inside_id 下
    :param pk:
    :param inside_id:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthPermission.move_inside(db=db, inside_id=inside_id, pk=pk)
    return Schemas(data=bool_model)


@router.patch("/{}/{{pk}}/move_after".format(permission_name), name="update {}".format(permission_name))
async def move_after_permission_model(pk: int, after_id: int, db: Session = Depends(dbs),
                                      auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.update" % permission_name])):
    """
    移动到 after_id 后
    :param pk:
    :param after_id:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthPermission.move_after(db=db, after_id=after_id, pk=pk)
    return Schemas(data=bool_model)


@router.delete("/{}/{{pk}}".format(permission_name), name="delete {}".format(permission_name))
async def delete_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.delete" % permission_name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthPermission.delete(db=db, pk=pk)
    return Schemas(data=bool_model)


@router.delete("/{}".format(permission_name), name="deletes {}".format(permission_name))
async def delete_permission_models(pks: List[int], db: Session = Depends(dbs),
                                   auth: SchemasOAuthScopes = Security(auth_user, scopes=permission_scopes + ["%s.delete" % permission_name])):
    """
    :param pks:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthPermission.delete(db=db, pks=pks)
    return Schemas(data=bool_model)



from typing import List

from fastapi import APIRouter, Depends, Security, HTTPException, status
from sqlalchemy.orm import Session

from config import OAUTH_DEFAULT_TAGS
from lsshu.internal.db import dbs
from lsshu.internal.depends import model_screen_params, auth_user
from lsshu.internal.schema import ModelScreenParams, Schemas
from lsshu.oauth.model import role_name
from lsshu.oauth.permission.crud import CRUDOAuthPermission
from lsshu.oauth.role.crud import CRUDOAuthRole
from lsshu.oauth.role.schema import SchemasOAuthRolePaginateItem, SchemasParams, SchemasOAuthRoleResponse, SchemasOAuthRoleStoreUpdate
from lsshu.oauth.user.schema import SchemasOAuthScopes

tags = OAUTH_DEFAULT_TAGS + ['Role']
router = APIRouter(tags=tags)
role_scopes = [role_name, ]


@router.get('/{}'.format(role_name), name="get {}".format(role_name))
async def get_models(db: Session = Depends(dbs), params: ModelScreenParams = Depends(model_screen_params),
                     auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.list" % role_name])):
    """
    :param db:
    :param params:
    :param auth:
    :return:
    """
    db_model_list = CRUDOAuthRole.paginate(db=db, screen_params=params)
    return Schemas(data=SchemasOAuthRolePaginateItem(**db_model_list))


@router.get('/{}.params'.format(role_name), name="get {}".format(role_name))
async def params_models(db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.list" % role_name])):
    """
    :param db:
    :param auth:
    :return:
    """

    def json_fields(node):
        return node.to_dict()

    data = {
        "permissions": CRUDOAuthPermission.get_tree(db=db, json=True, json_fields=json_fields)
    }
    return Schemas(data=SchemasParams(**data))


@router.get('/{}/{{pk}}'.format(role_name), name="get {}".format(role_name))
async def get_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.get" % role_name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthRole.first(db=db, pk=pk)
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="{} not found".format(role_name.capitalize()))
    return Schemas(data=SchemasOAuthRoleResponse(**db_model))


@router.post('/{}'.format(role_name), name="get {}".format(role_name))
async def store_model(item: SchemasOAuthRoleStoreUpdate, db: Session = Depends(dbs),
                      auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.store" % role_name])):
    """
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthRole.first(db=db, where=("name", item.name))
    if db_model is not None:
        raise HTTPException(status_code=status.HTTP_400_BAD_REQUEST,
                            detail="{} already registered".format(role_name.capitalize()))
    bool_model = CRUDOAuthRole.store(db=db, item=item)
    return Schemas(data=SchemasOAuthRoleResponse(**bool_model.to_dict()))


@router.put("/{}/{{pk}}".format(role_name), name="update {}".format(role_name))
async def update_put_model(pk: int, item: SchemasOAuthRoleStoreUpdate, db: Session = Depends(dbs),
                           auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.update" % role_name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthRole.first(db=db, pk=pk)
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="{} not found".format(role_name.capitalize()))
    bool_model = CRUDOAuthRole.update(db=db, pk=pk, item=item)
    return Schemas(data=SchemasOAuthRoleResponse(**bool_model.to_dict()))


@router.patch("/{}/{{pk}}".format(role_name), name="update {}".format(role_name))
async def update_patch_model(pk: int, item: SchemasOAuthRoleStoreUpdate, db: Session = Depends(dbs),
                             auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.update" % role_name])):
    """
    :param pk:
    :param item:
    :param db:
    :param auth:
    :return:
    """
    db_model = CRUDOAuthRole.first(db=db, pk=pk)
    if db_model is None:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="{} not found".format(role_name.capitalize()))
    bool_model = CRUDOAuthRole.update(db=db, pk=pk, item=item, exclude_unset=True)
    return Schemas(data=SchemasOAuthRoleResponse(**bool_model.to_dict()))


@router.delete("/{}/{{pk}}".format(role_name), name="delete {}".format(role_name))
async def delete_model(pk: int, db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.delete" % role_name])):
    """
    :param pk:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthRole.delete(db=db, pk=pk)
    return Schemas(data=bool_model)


@router.delete("/{}".format(role_name), name="deletes {}".format(role_name))
async def delete_role_models(pks: List[int], db: Session = Depends(dbs), auth: SchemasOAuthScopes = Security(auth_user, scopes=role_scopes + ["%s.delete" % role_name])):
    """
    :param pks:
    :param db:
    :param auth:
    :return:
    """
    bool_model = CRUDOAuthRole.delete(db=db, pks=pks)
    return Schemas(data=bool_model)



from datetime import datetime
from typing import Optional, List

from pydantic import BaseModel

from lsshu.internal.schema import Schemas, SchemasPaginate
from lsshu.oauth.permission.schema import SchemasOAuthPermissionResponse
from lsshu.oauth.role.schema import SchemasOAuthRoleResponse


class SchemasOAuthUserResponse(BaseModel):
    """授权用户 返回"""
    id: int
    username: Optional[str] = None
    permissions: Optional[List[SchemasOAuthPermissionResponse]] = None
    roles: Optional[List[SchemasOAuthRoleResponse]] = None
    stores: Optional[list] = None
    available: Optional[bool] = True
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None

    class Config:
        orm_mode = True


class SchemasPaginateItem(SchemasPaginate):
    items: List[SchemasOAuthUserResponse]


class SchemasLogin(BaseModel):
    access_token: Optional[str] = None
    token_type: Optional[str] = None


class SchemasLoginResponse(Schemas):
    """登录"""
    data = SchemasLogin


class SchemasOAuthUserMeStatusResponse(Schemas):
    """登录授权用户 状态返回"""
    data: SchemasOAuthUserResponse
    scopes: Optional[list] = None


class SchemasOAuthUserStoreUpdate(BaseModel):
    """授权用户 提交"""
    username: Optional[str] = None
    password: Optional[str] = None
    available: Optional[bool] = True
    permissions: Optional[List[int]] = None
    roles: Optional[List[int]] = None
    stores: Optional[List[int]] = None


class SchemasOAuthUser(BaseModel):
    """解析加密字段"""
    sub: Optional[str] = None
    user_id: Optional[int] = 0
    exp: Optional[int] = 0
    scopes: List[str] = []


class SchemasOAuthScopes(BaseModel):
    """验证授权后"""
    user: SchemasOAuthUserResponse
    scopes: List[str] = []


class SchemasParams(BaseModel):
    """参数"""
    roles: List[SchemasOAuthRoleResponse]
    permissions: List[SchemasOAuthPermissionResponse]



from sqlalchemy.orm import Session

from lsshu.internal.crud import CRUDTree
from lsshu.internal.helpers import token_get_password_hash
from lsshu.oauth.model import ModelOAuthUsers, ModelOAuthPermissions, ModelOAuthRoles
from lsshu.oauth.user.schema import SchemasOAuthUserStoreUpdate


class CRUDOAuthUser(CRUDTree):
    """用户表操作"""
    params_model = ModelOAuthUsers
    params_relationship = {
        "permissions": ModelOAuthPermissions,
        "roles": ModelOAuthRoles
    }

    @classmethod
    def store(cls, db: Session, item: SchemasOAuthUserStoreUpdate, **kwargs):
        if hasattr(item, "password") and item.password:
            item.password = token_get_password_hash(item.password)
        return super(CRUDOAuthUser, cls).store(db=db, item=item, **kwargs)

    @classmethod
    def update(cls, db: Session, pk: int, item: SchemasOAuthUserStoreUpdate, **kwargs):
        if hasattr(item, "password") and item.password:
            item.password = token_get_password_hash(item.password)
        else:
            if hasattr(item, "password"):
                delattr(item, "password")

        return super(CRUDOAuthUser, cls).update(db=db, pk=pk, item=item, **kwargs)


from sqlalchemy import Column, String, Boolean, Text
from sqlalchemy_mptt.mixins import BaseNestedSets

from lsshu.internal.db import Model, Engine


class _ModelOAuthUsers(Model, BaseNestedSets):
    """登录用户"""
    __abstract__ = True
    username = Column(String(15), nullable=False, unique=True, index=True, comment="名称")
    password = Column(String(128), nullable=False, comment="密码")
    available = Column(Boolean, default=1, comment="是否有效")


class _ModelOAuthRoles(Model, BaseNestedSets):
    """角色"""
    __abstract__ = True
    name = Column(String(15), nullable=False, unique=True, comment="名称")
    scopes = Column(Text, nullable=False, comment="Scope")


class _ModelOAuthPermissions(Model, BaseNestedSets):
    """ 权限 """
    __abstract__ = True
    name = Column(String(15), nullable=False, comment="名称")
    icon = Column(String(20), nullable=True, comment="ICO")
    scope = Column(String(50), nullable=False, unique=True, comment="Scope")
    path = Column(String(50), nullable=False, comment="Path")
    is_menu = Column(Boolean, default=True, comment="是否为菜单")
    is_action = Column(Boolean, default=True, comment="是否为动作权限")



"""
@Time    ：2022/5/10 10:55
@Author  ：Lsshu
@File    ：setup.py
@Version ：2.0.1
@Project ：lsshu-cms
"""
from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(name="lsshu-cms", version="2.0.1", description="FastAPI 开发的CMS", python_requires=">=3.8",
      author="Lsshu", author_email="admin@lsshu.cn", url="https://github.com/lsshu/fastapi-cms",
      packages=find_packages(), long_description=long_description,
      long_description_content_type="text/markdown", license="GPLv3",
      classifiers=[
          "Programming Language :: Python :: 3.8",
          "License :: OSI Approved :: MIT License",
          "Development Status :: 3 - Alpha",
      ],
      install_requires=["uvicorn[standard]", "fastapi", "sqlalchemy", "sqlalchemy_mptt", "python-multipart", "hashids", "passlib[bcrypt]", "python-jose[cryptography]"]
      )
